#define MOZ_SOURCE_STAMP 5bbf2e7c2fc6ff9010b1948e2f11477f48ee36e2
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/5bbf2e7c2fc6ff9010b1948e2f11477f48ee36e2
